/*
 * @seisFileListTable.java Version 1.1 03/04/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;

import gui.guiTable;

/** Class seisFileListTable
 *  <p> This Class will display the IQSTRAT Data Well List Table.
 *
 *  @version 1.1 03/04/2015
 *  @author  John R. Victorine
 */

public class seisFileListTable
{
  private int      iRows     = 0;
  private int      iColumns  = 1;
  private String   sColumn[] = { "MiniSeed File" };

  private Object   oData[][] = null;
  private guiTable pTable    = null;

  private int      iData     = 0;
  private String   sData[]   = null;

  private int      iChange   = 0;

  /** Constructor seisFileListTable()
   *  <p> This is the Constructor for this class.
   */

  public seisFileListTable()
  {
    pTable = new guiTable(0, iColumns, sColumn);
  }

  /** Constructor seisFileListTable( seisStruct st )
   *  <p> This is the Constructor for this class.
   *  @param sFile = Miniseed File name
   */

  public seisFileListTable( String sFile )
  {
    if (sFile != null)
    {
      sData = addData(sFile, iData, sData);
      if (sData != null)
        iData = sData.length;
    }

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
    sColumn    = null;
    sData      = null;
    oData      = null;
    pTable     = null;
  }

  /** Method populateList()
   *  <p> This method will populate the IQSTRAT List data.
   */

  private void populateList()
  {
	int i, j;
    int iCount = 0;

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (sData != null)
    {
      if (iData > 0)
      {
        iChange = 1;
        oData   = new Object[iData][iColumns];
        iCount  = 0;

        for (i=0; i<iData; i++)
        {
          oData[iCount][0]  = new String( sData[i] );
          iCount++;
        }

        iRows = iCount;
      }
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData( String sFile )
   *  <p> This method will reload the Region List after adding, deleting from
   *      the formation name list.
   * @param sFile = Miniseed File name
   */

  public void setData( String sFile )
  {
    if (sFile != null)
    {
      sData = addData(sFile, iData, sData);
      if (sData != null)
        iData = sData.length;
    }

    populateList();

    pTable.reload( iRows, oData );

    if (iChange == 0)
      iRows = 0;
  }

  /** Method addData()
   *  <p> This method will add the file identifier data into list.
   *  @param sFile = Miniseed File name
   *  @param sData = Array of Seismic Data File Identifier Data
   *  @param sData = Array of Seismic Data File Identifier Data
   */

  public String[] addData( String sFile, int iData, String sData[] )
  {
    int i, j;
    String stemp[] = null;
    int    iCount  = 0;

    if (sFile != null)
    {
      iData++;
      stemp = new String[iData];

      for (i=0; i<iData; i++)
      {
        stemp[iCount] = new String("");
      }

      if (iData > 1)
      {
        for (i=0; i<iData-1; i++)
        {
          stemp[iCount] = new String( sData[i] );
          iCount++;
        }
      }

      stemp[iCount] = new String(sFile);  // Network Code

      iCount++;

      sData = new String[iCount];

      for (i=0; i<iCount; i++)
      {
        sData[i] = new String( stemp[i] );
      }

      stemp = null;
    }

    return (sData);
  }

  /** Method clear()
   *  <p> This method will clear the contents of the table.
   */

  public void clear()
  {
    iData = 0;
    sData = null;

    populateList();

    pTable.reload( iRows, oData );

    if (iChange == 0)
      iRows = 0;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getCount()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getCount() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 03/04/2015
 *  @author  John Victorine
 */
