/*
 * @seisPlotXYFrame.java Version 1.1 03/16/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import gui.guiSearchDirectoryFrame;    // File Search Dialog

import seismic.plot.seisPlotXYData;
import seismic.seisPlotXYStruct;

import plot.plotSymbolStruct;

/** Class seisPlotXYFrame
 *  <p> This class will display the 2D Plot Frame
 *
 *  @version 1.1 03/16/2015
 *  @author  John R. Victorine
 */

public class seisPlotXYFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private Observable       notifier = null; // Observalbles Object
  private int              iPlot    = -1;
  private seisPlotXYStruct stPlot[] = null;
  private seisPlotXYData   panel    = null;

  // Global Variables

  // Save Path & Filename

  private String                  sPath   = new String("");
  private String                  sName   = new String("");
  private guiSearchDirectoryFrame pSearch = null; // Search for Directory to Save PNG

  private String      sTitle      = "";

  // X-Axis Values.

  public int          iXLog       = 0;   // X-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iXCycles    = 1;   // Number of Cycles
  public double       dXMaximum   = 0.0; // Maximum X-Axis
  public double       dXMinimum   = 0.0; // Minimum X-Axis
  public double       dXIncrement = 0.0; // Increment along the X-Axis
  public String       sXAxis      = "";  // X-Axis Label.
  private String      sXIncr      = "Increment:";

  // Y-Axis Values.

  public int          iYLog       = 0;   // Y-Axis Semilog Indicator
                                         //   1-Semilog
                                         //   0-Linear
  public int          iYCycles    = 1;   // Number of Cycles
  public double       dYMaximum   = 0.0; // Maximum Y-Axis
  public double       dYMinimum   = 0.0; // Minimum Y-Axis
  public double       dYIncrement = 0.0; // Increment along the Y-Axis
  public String       sYAxis      = "";  // Y-Axis Label.
  private String      sYIncr      = "Increment:";

  // Global Frame Widgets

  // .. Plot Titles

  private JTextField   txtTitle1  = new JTextField();
  private JTextField   txtTitle2  = new JTextField();
  private JTextField   txtTitle3  = new JTextField();

  // .. X-Axis Widgets

  private JTextField   txtXAxis   = new JTextField();
  private JTextField   txtXMin    = new JTextField();
  private JTextField   txtXMax    = new JTextField();
  private JTextField   txtXIncr   = new JTextField();

  // .. Y-Axis Widgets

  private JTextField   txtYAxis   = new JTextField();
  private JTextField   txtYMin    = new JTextField();
  private JTextField   txtYMax    = new JTextField();
  private JTextField   txtYIncr   = new JTextField();

  // -- Menus
  // .. File Button Menu

  private JMenuItem mPDF  = null; // Create a PDF of Plot
  private JMenuItem mExit = null; // Exit Plot

  /** Constructor seisPlotXYFrame()
   *  <p> This is the constructor for this class
   * @param notifier = Oberservable
   * @param iPlot    = Plot Type
   * @param stPlot   = XY Plot Data Structure
   */

  public seisPlotXYFrame( int iPlot, seisPlotXYStruct stPlot[] )
  {
    try
    {
	  this.iPlot    = iPlot;
      this.stPlot   = stPlot;

      if (iPlot > 0)
      {
		if (stPlot[0] != null)
		{
		  sTitle = new String(stPlot[0].sTitle1);
		  sXAxis = new String(stPlot[0].sXAxis);
		  if (stPlot[0].iXLog == 1) { sXIncr = new String("Cycles:"); }
		  sYAxis = new String(stPlot[0].sYAxis);
		  if (stPlot[0].iYLog == 1) { sYIncr = new String("Cycles:"); }

          // Y-Axis Values.

		  iXLog       = stPlot[0].iXLog;       // X-Axis Semilog Indicator
                                               //   1-Semilog
                                               //   0-Linear
		  iXCycles    = stPlot[0].iXCycles;    // Number of Cycles
		  dXMaximum   = stPlot[0].dXMaximum;   // Maximum X-Axis
		  dXMinimum   = stPlot[0].dXMinimum;   // Minimum X-Axis
		  dXIncrement = stPlot[0].dXIncrement; // Increment along the X-Axis
		  sXAxis      = new String(stPlot[0].sXAxis); // X-Axis Label.

          // Y-Axis Values.

		  iYLog       = stPlot[0].iYLog;       // Y-Axis Semilog Indicator
                                               //   1-Semilog
                                               //   0-Linear
		  iYCycles    = stPlot[0].iYCycles;    // Number of Cycles
		  dYMaximum   = stPlot[0].dYMaximum;   // Maximum Y-Axis
		  dYMinimum   = stPlot[0].dYMinimum;   // Minimum Y-Axis
		  dYIncrement = stPlot[0].dYIncrement; // Increment along the Y-Axis
		  sYAxis      = new String(stPlot[0].sYAxis); // Y-Axis Label.
		}
	  }

      jbInit();

      addWindowListener( new seisPlotXYFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel      pnlTop      = new JPanel();
	JPanel      pnlTitles   = new JPanel();
	JPanel      pnlAxes     = new JPanel();
    JScrollPane jScrollPane = new JScrollPane();

    JMenuBar    menuBar      = new JMenuBar();
    JMenu       menuFile     = new JMenu("File");

    // Build Menus

    // -- Menu Bar

    mPDF = new JMenuItem("Create PDF Document Plot");
    mPDF.addActionListener(this);

    mExit = new JMenuItem("Exit");
    mExit.addActionListener(this);

    menuFile.add( mPDF );
    menuFile.addSeparator();
    menuFile.add( mExit );

    menuBar.add(menuFile);
    this.setJMenuBar(menuBar);

    // Create an Observable

    notifier = new seisPlotXYFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle( sTitle );

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    pnlTitles = plotTitlesPanel();

    pnlAxes   = plotAxesPanel();

    // Center Panel

    panel = new seisPlotXYData(iPlot, stPlot);

    // Attach Widgets

    this.getContentPane().add(pnlTop,      BorderLayout.NORTH);
    pnlTop.add(pnlTitles,                  BorderLayout.NORTH);
    pnlTop.add(pnlAxes,                    BorderLayout.SOUTH);

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel,   null);

    // Set Dimension of Frame and Display it.

    this.setSize(new Dimension(900, 800));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setLocation( (d.width - this.getSize().width)/2,
    this.setLocation( (d.width - this.getSize().width) - 10,
                      (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method plotTitlesPanel()
   *  <p> This method will build the Plot Title Panels
   */

  private JPanel plotTitlesPanel()
  {
	JPanel panel           = new JPanel();
    JPanel pnlTitles       = new JPanel();
    JPanel pnlTitle1       = new JPanel();
    JPanel pnlTitle2       = new JPanel();
    JPanel pnlTitle3       = new JPanel();

    TitledBorder titledBorderTitle1 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "1st Title:");
    titledBorderTitle1.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle2 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "2nd Title:");
    titledBorderTitle2.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderTitle3 = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "3rd Title:");
    titledBorderTitle3.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    String sTitle1 = "";
    String sTitle2 = "";
    String sTitle3 = "";

    if (iPlot > 0)
    {
	  if (stPlot[0] != null)
	  {
		sTitle1 = new String(stPlot[0].sTitle1);
		sTitle2 = new String(stPlot[0].sTitle2);
		sTitle3 = new String(stPlot[0].sTitle3);
	  }
	}

    panel.setLayout(new BorderLayout());

    pnlTitles.setLayout(new GridLayout());

    // .. First Title Panel

    pnlTitle1.setLayout(new BorderLayout());
    pnlTitle1.setBorder(titledBorderTitle1);

    txtTitle1.setText(sTitle1);
    txtTitle1.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle1.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. Second Title Panel

    pnlTitle2.setLayout(new BorderLayout());
    pnlTitle2.setBorder(titledBorderTitle2);

    txtTitle2.setText(sTitle2);
    txtTitle2.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle2.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. Third Title Panel

    pnlTitle3.setLayout(new BorderLayout());
    pnlTitle3.setBorder(titledBorderTitle3);

    txtTitle3.setText(sTitle3);
    txtTitle3.setHorizontalAlignment(SwingConstants.LEFT);
    txtTitle3.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // Attach Widgets

    panel.add(pnlTitles,                  BorderLayout.NORTH);
    pnlTitles.add(pnlTitle1,              null);
    pnlTitle1.add(txtTitle1,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle2,              null);
    pnlTitle2.add(txtTitle2,              BorderLayout.CENTER);

    pnlTitles.add(pnlTitle3,              null);
    pnlTitle3.add(txtTitle3,              BorderLayout.CENTER);

    return (panel);
  }

  /** Method plotAxesPanel()
   *  <p> This method will build the Plot Axis Panels
   */

  private JPanel plotAxesPanel()
  {
	JPanel panel           = new JPanel();
    JPanel pnlTop          = new JPanel();

    JPanel pnlAxis         = new JPanel();
    JPanel pnlXAxis        = new JPanel();
    JPanel pnlXAxisRange   = new JPanel();
    JPanel pnlXMin         = new JPanel();
    JPanel pnlXMax         = new JPanel();
    JPanel pnlXIncr        = new JPanel();

    JPanel pnlYAxis        = new JPanel();
    JPanel pnlYAxisRange   = new JPanel();
    JPanel pnlYMin         = new JPanel();
    JPanel pnlYMax         = new JPanel();
    JPanel pnlYIncr        = new JPanel();

    JLabel lblXAxis        = new JLabel();
    JLabel lblXMin         = new JLabel();
    JLabel lblXMax         = new JLabel();
    JLabel lblXIncr        = new JLabel();

    JLabel lblYAxis        = new JLabel();
    JLabel lblYMin         = new JLabel();
    JLabel lblYMax         = new JLabel();
    JLabel lblYIncr        = new JLabel();

    String sXAxis          = "";
    double dXMin           = 0.0;
    double dXMax           = 0.0;
    double dXIncr          = 0.0;

    String sYAxis          = "";
    double dYMin           = 0.0;
    double dYMax           = 0.0;
    double dYIncr          = 0.0;

    TitledBorder titledBorderMin = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum:");
    titledBorderMin.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum:");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderXIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        sXIncr);
    titledBorderXIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        sYIncr);
    titledBorderYIncr.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderXAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "X-Axis:");
    titledBorderXAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYAxis = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Y-Axis:");
    titledBorderYAxis.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    if (iPlot > 0)
    {
	  if (stPlot[0] != null)
	  {
		sXAxis = new String(stPlot[0].sXAxis);
		dXMin  = stPlot[0].dXMinimum;
		dXMax  = stPlot[0].dXMaximum;
		dXIncr = stPlot[0].dXIncrement;
		if (stPlot[0].iXLog == 1)
		  dXIncr = stPlot[0].iXCycles;

		sYAxis = new String(stPlot[0].sYAxis);
		dYMin  = stPlot[0].dYMinimum;
		dYMax  = stPlot[0].dYMaximum;
		dYIncr = stPlot[0].dYIncrement;
		if (stPlot[0].iYLog == 1)
		  dYIncr = stPlot[0].iYCycles;
	  }
	}

    // Top Panel

    panel.setLayout(new BorderLayout());

    // -- Axis Panel

    pnlAxis.setLayout(new GridLayout());

    // .. X-Axis Panel

    pnlXAxis.setLayout(new BorderLayout());
    pnlXAxis.setBorder(titledBorderXAxis);

//    lblXAxis.setText( "X-Axis" );

    txtXAxis.setText( sXAxis );
    txtXAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtXAxis.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. X-Axis Range Panel

    pnlXAxisRange.setLayout(new GridLayout());

    // .. .. X-Axis Minimum Panel

    pnlXMin.setLayout(new BorderLayout());
//    pnlXMin.setBorder(titledBorderMin);

    lblXMin.setText("Minimum: ");

    txtXMin.setText("" + dXMin);
    txtXMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMin.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. .. X-Axis Maximum Panel

    pnlXMax.setLayout(new BorderLayout());
//    pnlXMax.setBorder(titledBorderMax);

    lblXMax.setText(" Maximum: ");

    txtXMax.setText("" + dXMax);
    txtXMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXMax.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. .. X-Axis Increment Panel

    pnlXIncr.setLayout(new BorderLayout());
//    pnlXIncr.setBorder(titledBorderXIncr);

    lblXIncr.setText(" " + sXIncr +": ");

    txtXIncr.setText("" + dXIncr);
    txtXIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtXIncr.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    if (iXLog == 0)
      txtXMax.setEnabled(true);
    else
      txtXMax.setEnabled(false);

    // .. Y-Axis Panel

    pnlYAxis.setLayout(new BorderLayout());
    pnlYAxis.setBorder(titledBorderYAxis);

    txtYAxis.setText( sYAxis );
    txtYAxis.setHorizontalAlignment(SwingConstants.LEFT);
    txtYAxis.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. Y-Axis Range Panel

    pnlYAxisRange.setLayout(new GridLayout());

    // .. .. Y-Axis Minimum Panel

    pnlYMin.setLayout(new BorderLayout());
//    pnlYMin.setBorder(titledBorderMin);

    lblYMin.setText("Maximum: ");

    txtYMin.setText("" + dYMin);
    txtYMin.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMin.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. .. Y-Axis Maximum Panel

    pnlYMax.setLayout(new BorderLayout());
//    pnlYMax.setBorder(titledBorderMax);

    lblYMax.setText(" Maximum: ");

    txtYMax.setText("" + dYMax);
    txtYMax.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYMax.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    // .. .. Y-Axis Increment Panel

    pnlYIncr.setLayout(new BorderLayout());
//    pnlYIncr.setBorder(titledBorderYIncr);

    lblYIncr.setText(" " + sYIncr +": ");

    txtYIncr.setText("" + dYIncr);
    txtYIncr.setHorizontalAlignment(SwingConstants.RIGHT);
    txtYIncr.addFocusListener(new seisPlotXYFrameFocusAdapter(this));

    if (iYLog == 0)
      txtYMax.setEnabled(true);
    else
      txtYMax.setEnabled(false);

    // Attach Widgets

    panel.add(pnlAxis,                    BorderLayout.NORTH);

    pnlAxis.add(pnlXAxis,                 null);
    pnlXAxis.add(txtXAxis,                BorderLayout.NORTH);

    pnlXAxis.add(pnlXAxisRange,           BorderLayout.SOUTH);
    pnlXAxisRange.add(pnlXMin,            null);
    pnlXMin.add(lblXMin,                  BorderLayout.WEST);
    pnlXMin.add(txtXMin,                  BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXMax,            null);
    pnlXMax.add(lblXMax,                  BorderLayout.WEST);
    pnlXMax.add(txtXMax,                  BorderLayout.CENTER);

    pnlXAxisRange.add(pnlXIncr,           null);
    pnlXIncr.add(lblXIncr,                BorderLayout.WEST);
    pnlXIncr.add(txtXIncr,                BorderLayout.CENTER);

    pnlAxis.add(pnlYAxis,                 null);
    pnlYAxis.add(txtYAxis,                BorderLayout.NORTH);

    pnlYAxis.add(pnlYAxisRange,           BorderLayout.SOUTH);
    pnlYAxisRange.add(pnlYMin,            null);
    pnlYMin.add(lblYMin,                  BorderLayout.WEST);
    pnlYMin.add(txtYMin,                  BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYMax,            null);
    pnlYMax.add(lblYMax,                  BorderLayout.WEST);
    pnlYMax.add(txtYMax,                  BorderLayout.CENTER);

    pnlYAxisRange.add(pnlYIncr,           null);
    pnlYIncr.add(lblYIncr,                BorderLayout.WEST);
    pnlYIncr.add(txtYIncr,                BorderLayout.CENTER);

    return (panel);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD seisPlotXYFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class seisPlotXYFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  if (notifier != null)
        notifier.notifyObservers(new String("Close Plot Control"));
    }
  }

  /** Method close()
   *  <p> This method will deallocate the global variables to force Java
   *      to free allocated memory.
   */

  public void close()
  {
    stPlot    = null;

    if (panel != null)
      panel.close();
    panel     = null;

    if (pSearch != null)
      pSearch.close();
    pSearch   = null;

    sPath   = null;
    sName   = null;

    sTitle    = null;
    sXAxis    = null;
    sXIncr    = null;
    sYAxis    = null;
    sYIncr    = null;

    // Global Frame Widgets

    // .. Plot Titles

    txtTitle1 = null;
    txtTitle2 = null;
    txtTitle3 = null;

    // .. X-Axis Widgets

    txtXAxis  = null;
    txtXMin   = null;
    txtXMax   = null;
    txtXIncr  = null;

    // .. Y-Axis Widgets

    txtYAxis  = null;
    txtYMin   = null;
    txtYMax   = null;
    txtYIncr  = null;

    // -- Menus

    mPDF      = null; // PDF of Profile Plot
    mExit     = null; // Exit

    dispose();
  }

  /* ================================================================== *
   * -------------------------- GET METHODS --------------------------- *
   * ================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /* ================================================================== *
   * -------------------------- SET METHODS --------------------------- *
   * ================================================================== */

  /** Method refresh()
   * <p> This method will refresh the plot if it exists
   */

  public void refresh() { panel.repaint(); }

  /** Method setLegend()
   *  <p> The Legend for the XY Plot.
   *  @param iLegend  - Number of Legend Rows.
   *  @param stLegend - Array of Legends.
   */

  public void setLegend(int iLegend, plotSymbolStruct stLegend[])
  {
    panel.setLegend(iLegend, stLegend);
  }

  /** Method setFlag()
   *  <p> Set a Flag to filter extra data or not.
   *  @param bFlag = true to filter data or false not to filter extra data.
   */

  public void setFlag(boolean bFlag) { panel.setFlag(bFlag); }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
	txtTitle1.setText(sText);
	panel.setTitle1(sText);
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
	txtTitle2.setText(sText);
	panel.setTitle2(sText);
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
	txtTitle3.setText(sText);
	panel.setTitle3(sText);
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setXAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setYAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setData()
   *  <p> This method will set the data to be plotted
   *  @param iData = The data type to be plotted
   */

  public void setData(int iData) { panel.setData(iData); }

  /* ===================================================================== *
   * -------------------------- PDF ACTIONS ------------------------------ *
   * ===================================================================== */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Production Plot
   */

  private void createPDF()
  {
    seismic.io.seisIO_Files.createImage( sPath, sName, panel.getImage() );
    seismic.io.seisIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == mPDF) // Create a PDF of Profile Plot
    {
	  if (sPath.length() == 0)
        sPath = seismic.io.seisIO_Files.getHomeDir();

      if (sName.length() == 0)
        sName = new String( "seismic_XY" );

      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

    if (event.getSource() == mExit)  { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	focusLostTitles(e);
	focusLostXAxes(e);
	focusLostYAxes(e);
  }

  /** METHOD focusLostTitles()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTitles(FocusEvent e)
  {
    if (e.getSource() == txtTitle1)
    {
	  setTitle1(txtTitle1.getText());
	}

    if (e.getSource() == txtTitle2)
    {
	  setTitle2(txtTitle2.getText());
	}

    if (e.getSource() == txtTitle3)
    {
	  setTitle3(txtTitle3.getText());
	}
  }

  /** METHOD focusLostXAxes()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostXAxes(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    String s1       = "";
    String s2       = "";
    String s3       = "";
    double d1       = 0.0;
    double d2       = 0.0;
    double dTemp    = 0.0;
    int    iMin     = 0;
    int    iMax     = 0;
    int    iDiff    = 0;
    double dMin     = 0.0;
    double dMax     = 0.0;
    double diff     = 0.0;

    if ((e.getSource() == txtXAxis) ||
        (e.getSource() == txtXMin ) ||
        (e.getSource() == txtXMax ) ||
        (e.getSource() == txtXIncr))
    {
      if (e.getSource() == txtXAxis)
      {
        sXAxis = new String( txtXAxis.getText() );
        setXAxis( sXAxis, iXLog, dXMaximum, dXMinimum, dXIncrement, iXCycles );
      }

      if (e.getSource() == txtXMin )
      {
        iNumeric = 1;
        sTemp    = txtXMin.getText();
        sMessage = new String("X-Minimum Value is a Numeric Field");

        s1 = txtXMin.getText();
        s2 = txtXMax.getText();

        if (s1.equals(s2))
        {
          txtXMin.setText(""+dXMinimum);

	      sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
        }

        if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
        {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

          if (d1 > d2 )
      	  {
      	    txtXMin.setText(""+dXMinimum);

    	    sTemp = new String("A");
            sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
    	  }
        }
      }

      if (e.getSource() == txtXMax )
      {
        iNumeric = 1;
        sTemp    = txtXMax.getText();
        sMessage = new String("X-Maximum Value is a Numeric Field");

        s1 = txtXMin.getText();
        s2 = txtXMax.getText();

        if (s1.equals(s2))
        {
	      txtXMax.setText(""+dXMaximum);

	      sTemp = new String("A");
          sMessage = new String("X-Minimum Value can not equal X-Maximum Value");
        }

        if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
        {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	        txtXMax.setText(""+dXMaximum);

	        sTemp = new String("A");
            sMessage = new String("X-Minimum Value can not be greater than X-Maximum Value");
      	  }
        }
      }

      if (e.getSource() == txtXIncr)
      {
        iNumeric = 1;
        sTemp    = txtXIncr.getText();
        sMessage = new String("X-Increment Value is a Numeric Field");

        s3 = txtXIncr.getText();
        if ((s3.equals("0.0")) || (s3.equals("0")))
        {
    	  txtXIncr.setText(""+dXIncrement);
    	  if (iXLog == 1) { txtYIncr.setText(""+iXCycles); }
    	  sTemp = new String("A");
          sMessage = new String("X-Increment Value can not be 0");
        }
      }

      if (iNumeric == 1)
      {
        if (!cmn.cmnString.isNumeric(sTemp))
        {
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
        }
        else
        {
          if (e.getSource() == txtXMin )
          {
            dXMinimum = cmn.cmnString.stringToDouble(txtXMin.getText());

    	    if (iXLog == 1)
	 	    {
              dMin = Math.log(dXMinimum)/Math.log(10.0);
              if (dMin < 0.0) { dMin = dMin - 1.0; }
              iMin = (int) dMin;
              dXMinimum = Math.pow(10.0, iMin);
              txtXMin.setText(""+dXMinimum);

              iMax = iMin + iXCycles;
              dXMaximum = Math.pow(10.0, iMax);
              txtXMax.setText(""+dXMaximum);
	        }
          }

          if (e.getSource() == txtXMax )
            dXMaximum = cmn.cmnString.stringToDouble(txtXMax.getText());

          if (e.getSource() == txtXIncr )
          {
		    dTemp = cmn.cmnString.stringToDouble(txtXIncr.getText());
		    if (iXLog == 0)
		      dXIncrement = dTemp;
		    else
		    {
		      iXCycles    = (int) dTemp;
		      txtXIncr.setText(""+iXCycles);
	        }

		    if (iXLog == 1)
		    {
              iMin = (int) ( Math.log(dXMinimum)/Math.log(10.0) );
              iMax = iMin + iXCycles;
              dXMaximum = Math.pow(10.0, iMax);
              txtXMax.setText(""+dXMaximum);
            }
	      }

          setXAxis( sXAxis, iXLog, dXMaximum, dXMinimum, dXIncrement, iXCycles );
        }
	  }
	}
  }

  /** METHOD focusLostYAxes()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostYAxes(FocusEvent e)
  {
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");
    String s1       = "";
    String s2       = "";
    String s3       = "";
    double d1       = 0.0;
    double d2       = 0.0;
    double dTemp    = 0.0;
    int    iMin     = 0;
    int    iMax     = 0;
    int    iDiff    = 0;
    double dMin     = 0.0;
    double dMax     = 0.0;
    double diff     = 0.0;

    if ((e.getSource() == txtYAxis) ||
        (e.getSource() == txtYMin ) ||
        (e.getSource() == txtYMax ) ||
        (e.getSource() == txtYIncr))
    {
      if (e.getSource() == txtYAxis)
      {
        sYAxis = new String( txtYAxis.getText() );
        setYAxis( sYAxis, iYLog, dYMaximum, dYMinimum, dYIncrement, iYCycles );
	  }

      if (e.getSource() == txtYMin )
      {
        iNumeric = 1;
        sTemp    = txtYMin.getText();
        sMessage = new String("Y-Minimum Value is a Numeric Field");

        s1 = txtYMin.getText();
        s2 = txtYMax.getText();

        if (s1.equals(s2))
        {
	      txtYMin.setText(""+dYMinimum);

	      sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	    }

	    if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	    {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	     	  txtYMin.setText(""+dYMinimum);

	    	  sTemp = new String("A");
              sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
	       }
	    }
	  }

      if (e.getSource() == txtYMax )
      {
        iNumeric = 1;
        sTemp    = txtYMax.getText();
        sMessage = new String("Y-Maximum Value is a Numeric Field");

        s1 = txtYMin.getText();
        s2 = txtYMax.getText();

        if (s1.equals(s2))
        {
	      txtYMax.setText(""+dYMaximum);

	      sTemp = new String("A");
          sMessage = new String("Y-Minimum Value can not equal Y-Maximum Value");
	    }

	    if ((cmn.cmnString.isNumeric(s1)) && (cmn.cmnString.isNumeric(s2)))
	    {
	      d1 = cmn.cmnString.stringToDouble( s1 );
	      d2 = cmn.cmnString.stringToDouble( s2 );

	      if (d1 > d2 )
	      {
	        txtYMax.setText(""+dYMaximum);

	        sTemp = new String("A");
            sMessage = new String("Y-Minimum Value can not be greater than Y-Maximum Value");
	      }
	    }
	  }

      if (e.getSource() == txtYIncr)
      {
        iNumeric = 1;
        sTemp    = txtYIncr.getText();
        sMessage = new String("Y-Increment Value is a Numeric Field");

        s3 = txtYIncr.getText();
	    if ((s3.equals("0.0")) || (s3.equals("0")))
	    {
	   	  txtYIncr.setText(""+dYIncrement);
	      if (iYLog == 1) { txtYIncr.setText(""+iYCycles); }
	   	  sTemp = new String("A");
          sMessage = new String("Y-Increment Value can not be 0");
	    }
	  }

      if (iNumeric == 1)
      {
        if (!cmn.cmnString.isNumeric(sTemp))
        {
          JOptionPane.showMessageDialog((Component) null,
                                         sMessage,
                                         "ERROR",
                                         JOptionPane.ERROR_MESSAGE);
        }
        else
        {
          if ((e.getSource() == txtYMin ) ||
              (e.getSource() == txtYMax ) ||
              (e.getSource() == txtYIncr ))
          {
            if (e.getSource() == txtYMin )
            {
              dYMinimum = cmn.cmnString.stringToDouble(txtYMin.getText());

	   	      if (iYLog == 1)
	 	      {
                dMin = Math.log(dYMinimum)/Math.log(10.0);
                if (dMin < 0.0) { dMin = dMin - 1.0; }
                iMin = (int) dMin;
                dYMinimum = Math.pow(10.0, iMin);
                txtYMin.setText(""+dYMinimum);

                iMax = iMin + iYCycles;
                dYMaximum = Math.pow(10.0, iMax);
                txtYMax.setText(""+dYMaximum);
	          }
	        }

            if (e.getSource() == txtYMax )
              dYMaximum = cmn.cmnString.stringToDouble(txtYMax.getText());

            if (e.getSource() == txtYIncr )
            {
	          dTemp = cmn.cmnString.stringToDouble(txtYIncr.getText());
		      if (iYLog == 0)
		        dYIncrement = dTemp;
		      else
		      {
		        iYCycles    = (int) dTemp;
		        txtYIncr.setText(""+iYCycles);
	          }

		      if (iYLog == 1)
		      {
                iMin = (int) ( Math.log(dYMinimum)/Math.log(10.0) );
                iMax = iMin + iYCycles;
                dYMaximum = Math.pow(10.0, iMax);
                txtYMax.setText(""+dYMaximum);
	          }
	        }

            setYAxis( sYAxis, iYLog, dYMaximum, dYMinimum, dYIncrement, iYCycles );
		  }
        }
	  }
	}
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS seisPlotXYFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class seisPlotXYFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  seisPlotXYFrame adaptee;

  seisPlotXYFrameFocusAdapter(seisPlotXYFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS seisPlotXYFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class seisPlotXYFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 03/16/2015
 *  @author  John Victorine
 */
