/*
 * @seisStartAppPanel.java Version 1.1 03/04/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import cmn.cmnStruct;              // Common Data Structure
import seismic.gui.seisStartFrame; // Seismic Start Frame

/** Class seisStartAppPanel
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 03/04/2015
 *  @author  John R. Victorine
 */

public class seisStartAppPanel extends JPanel implements ActionListener
{
  // Input Variables

  private JFrame         parent = null; // Frame calling this panel
  private cmnStruct      stCMN  = null; // Common Data Structure

  // Global Variable

  private seisStartFrame frame  = null; // Seismic Start Frame

  // Global Panel Widgets

  private JButton    btn      = null;
  private JButton    btnExit  = new JButton();

  /** Constructor seisStartAppPanel( JFrame parent, cmnStruct stCMN )
   *  <p> This is the Constructor for this class.
   *  @param parent   = the frame calling this panel
   *  @param stCMN = Global IQSTRAT Data Structure
   */

  public seisStartAppPanel( JFrame parent, cmnStruct stCMN )
  {
    try
    {
      this.parent   = parent;
      this.stCMN = stCMN;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlCenter = new JPanel();
    JPanel pnlBase   = new JPanel();

    JLabel lbl       = new JLabel();

    // Build Panel

    this.setLayout( new BorderLayout() );
    this.setBackground(Color.white);

    pnlCenter.setLayout( new BorderLayout() );
    pnlCenter.setBackground(Color.white);

    btn = new JButton( createImageIcon( cmn.cmnStruct.SEISMIC_APP) );
    btn.setBackground(Color.white);
    btn.addActionListener( this );

    lbl.setHorizontalAlignment(SwingConstants.CENTER);
    lbl.setFont(new java.awt.Font("Dialog", 1, 20));
    lbl.setBackground(Color.white);
    lbl.setText("Enter");

    pnlBase.setLayout( new GridLayout() );
    pnlBase.setBackground(Color.white);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExit.setText("Exit");
    btnExit.addActionListener(this);

    // Add Widgets to Panel

    this.add( pnlCenter,    BorderLayout.CENTER );
    pnlCenter.add( btn,     BorderLayout.CENTER );
    pnlCenter.add( lbl,     BorderLayout.SOUTH );

    if (parent != null)
    {
      this.add( pnlBase,    BorderLayout.SOUTH );
      pnlBase.add( btnExit, null );
    }
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN = null;  // Global IQSTRAT Data Structure

    if (frame != null)
      frame.close();
    frame    = null;

    // Global Panel Widgets

    btn      = null;
    btnExit  = null;

    if (parent != null)
    {
      parent.dispose();
      parent = null;

      System.exit(0);
    }
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btn)
    {
      if (frame != null)
        frame.close();
      frame    = null;

      frame = new seisStartFrame( stCMN );
	}

    if (event.getSource() == btnExit)
    {
	  close();
	}
  }
}

/*
 *  @version 1.1 03/04/2015
 *  @author  John Victorine
 */
