/*
 * @seisStartFrame.java Version 1.1 03/06/2014
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import cmn.cmnStruct;			      // Global Common Data Structure
import seismic.seisStruct;            // Seismic Data Structure
import seismic.io.seisReadMSeed;      // Seismic Read & Parse miniseed File
import seismic.gui.seisHeaderTable;   // Seismic File Header Data Table
import seismic.gui.seisFileListTable; // Seismic File List Table
import seismic.gui.seisControlFrame;  // Seismic Plot Control Frame

/** Class seisStartFrame
 *  <p> This Class Starting Panel for this web application
 *
 *  @version 1.1 03/06/2014
 *  @author  John R. Victorine
 */

public class seisStartFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private cmnStruct  stCMN    = null;  // Common Data Structure

  // Global Variables

  private Observable notifier = null;  // Observables Object

  private seisStruct        stData  = null; // Seismic Data Structure
  private seisHeaderTable   pTable  = null; // Miniseed File Header Data Table
  private seisFileListTable pFTable = null; // Miniseed File List Table

  private seisControlFrame  control = null; // Seismic Plot Control Frame

  // Global Frame Widgets

  // -- Read miniseed File Icon Button

  private JButton btnPC       = null;  // Read MiniSeed File Icon

  // -- Frame Button Widgets

  private JButton btnContinue = new JButton(); // Plot seismic data
  private JButton btnClear    = new JButton(); // Clear all data
  private JButton btnExit     = new JButton(); // Exit this dialog

  /** Constructor seisStartFrame( cmnStruct stCMN )
   *  <p> This is the Constructor for this class.
   *  @param stCMN = Global IQSTRAT Data Structure
   */

  public seisStartFrame( cmnStruct stCMN )
  {
    try
    {
      this.stCMN = stCMN;

      jbInit();
      setButtons();

      addWindowListener( new seisStartFrame_WindowListener() );
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop    = new JPanel();
    JPanel pnlCenter = new JPanel();
    JPanel pnlBase   = new JPanel();

    JPanel pnlFPanel = new JPanel();

    JPanel pnlPC     = new JPanel();
    JPanel pnlPCicon = new JPanel();

    JScrollPane scroll     = new JScrollPane();
    JScrollPane scrollList = new JScrollPane();

    TitledBorder titledBorderPCIcon = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Read File");
    titledBorderPCIcon.setTitleJustification(TitledBorder.CENTER);
    titledBorderPCIcon.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Create an Observable

    notifier = new seisStartFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.setTitle( "Load MiniSeed File" );
    this.getContentPane().setLayout( new BorderLayout() );

    pnlTop.setLayout( new BorderLayout() );
    pnlTop.setPreferredSize( new Dimension(10, 120) );

    // -- PC Data Source

    pnlPC.setLayout( new BorderLayout() );

    // .. PC ICON Panel

    pnlPCicon.setBorder( titledBorderPCIcon );

    btnPC = new JButton( createImageIcon( cmn.cmnStruct.WAVE_ICON ) );
    btnPC.setBackground(Color.white);
    btnPC.setPreferredSize( new Dimension(64, 64) );
    btnPC.addActionListener( this );

    // .. Miniseed File List Table Panel

    pnlFPanel.setLayout( new BorderLayout() );

    pFTable    = new seisFileListTable();
    scrollList = pFTable.getScrollPane();

    // -- File Contents Identifer Table

    pnlCenter.setLayout( new BorderLayout() );

    pTable = new seisHeaderTable();
    scroll = pTable.getScrollPane();

    // -- Frame Buttons

    btnContinue.setFont(new java.awt.Font("Dialog", 1, 11));
    btnContinue.setPreferredSize( new Dimension(120, 25) );
    btnContinue.setText("Continue");
    btnContinue.addActionListener(this);

    btnClear.setFont(new java.awt.Font("Dialog", 1, 11));
    btnClear.setPreferredSize( new Dimension(120, 25) );
    btnClear.setText("Clear");
    btnClear.addActionListener(this);

    btnExit.setFont(new java.awt.Font("Dialog", 1, 11));
    btnExit.setPreferredSize( new Dimension(120, 25) );
    btnExit.setText("Exit");
    btnExit.addActionListener(this);

    // Add Widgets to Panel

    this.getContentPane().add( pnlTop,    BorderLayout.NORTH );
    pnlTop.add( pnlPC,                    BorderLayout.WEST );
    pnlPC.add( pnlPCicon,                 BorderLayout.CENTER );
    pnlPCicon.add( btnPC,                 null );

    pnlTop.add( pnlFPanel,                BorderLayout.CENTER);
    pnlFPanel.add(scrollList,             BorderLayout.CENTER);

    this.getContentPane().add( pnlCenter, BorderLayout.CENTER );
    pnlCenter.add(scroll,                 BorderLayout.CENTER);

    this.getContentPane().add( pnlBase,   BorderLayout.SOUTH );
    pnlBase.add( btnContinue,             null );
    pnlBase.add( btnClear,                null );
    pnlBase.add( btnExit,                 null );

    // Display the Frame

    this.setSize(new Dimension(665, 300));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** METHOD seisStartFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class seisStartFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e) { close(); }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN = null; // Global IQSTRAT Data Structure

    // Global Variables

    notifier = null; // Observables Object

    if (stData != null)
      stData.delete();
    stData   = null;  // Seismic Data Structure

    if (pTable != null)
      pTable.close();
    pTable = null;    // Seismic File List Table

    // Global Frame Widgets

    btnPC       = null; // PC Icon  - No Action

    btnContinue = null; // Continue
    btnClear    = null; // Clear
    btnExit     = null; // Exit

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will set the Profile Plot buttons depending on the
   *      selected data.
   */

  private void setButtons()
  {
    btnContinue.setEnabled(false);
    btnPC.setEnabled(true);

    if (pTable.getCount() > 0)
    {
      btnContinue.setEnabled(true);
      btnPC.setEnabled(false);
	}
  }

  /** Method clear()
   *  <p> This method will clear all data
   */

  private void clear()
  {
    if (stData != null)
      stData.delete();
    stData   = null;  // Seismic Data Structure

    pTable.clear();
    pFTable.clear();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /* ===================================================================== *
   * ------------------- READ MINISEED FILE METHOD ----------------------- *
   * ===================================================================== */

  /** Method readFile()
   *  <p> This method will read the miniseed file into this dialog
   */

  private void readFile()
  {
	seisReadMSeed pIn       = null;
    FileDialog    dialog    = null;
    Frame         frameTemp = new Frame();

    // Open Directory Search Dialog to locate miniseed File Location

    dialog = new FileDialog( frameTemp, "Select MiniSeed File from your PC." );
    dialog.setVisible( true );
    frameTemp.setVisible( true );

    // Retrieve miniseed File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      String sFile = new String(dialog.getDirectory() + "/" + dialog.getFile());

	  // Input miniseed data from file into seismic data structure

      pIn    = new seisReadMSeed();
      pIn.Open(sFile);                                        // Open File
      stData = seismic.seisUtility.copyData( pIn.getData() ); // Get Data
      pIn.Close();                                            // Close File

      pFTable.setData( dialog.getFile() );
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;

    // Add Header Information to File List Table

    pTable.setData( stData );
    setButtons();
  }

  /* ===================================================================== *
   * -------------------------- PLOT METHOD ------------------------------ *
   * ===================================================================== */

  /** Method plot()
   *  <p> This method will plot the data
   */

  private void plot()
  {
	if (control != null)
	  control.close();
	control = null;

	control = new seisControlFrame(stCMN);
	control.setData( stData );
  }
/*
  private void plot()
  {
	int i=0;
	String    sDate  = "";
	String    sTime  = "";
    Timestamp ts     = null;
    Timestamp tsTime = null;
    java.sql.Date      date   = null;
    long      ldate  = (long) 0;
    long      ldiff;
    long      ltime  = (long) 0;
    String    str        = "";
    String    tokens[]   = null;
    String    sDelimiter = new String("[ ]+");

    int iIncrementY = 100;  // Increment Height
    int iScale      = seismic.seisTracksStruct.SCALE_5;
    SimpleDateFormat dfm = new SimpleDateFormat("yyyy-D-MM HH:mm:ss.SSSSSS");
//    SimpleDateFormat dfm = new SimpleDateFormat("yyyy-D-MM HH:mm:ss");

      long lstart     = stData.tsStart.getTime();
      long lend       = stData.tsEnd.getTime();
      long diff       = lend-lstart;

      System.out.println(lstart+" "+lend+" "+diff);

      int    iLogHeight     = (int) ((100 * (double) diff) /
                                seismic.seisTracksStruct.SCALE[iScale]);
      double dIncrementTime = seismic.seisTracksStruct.SCALE[iScale];


      System.out.println("Start="+stData.tsStart.toString());
      for (i=0; i<=iLogHeight; i+=iIncrementY)
      {
        ltime = (long) (lstart +
                      (long)(i * dIncrementTime / (double) iIncrementY));
		tsTime = new Timestamp( ltime );
		System.out.println(i+" "+tsTime.toString());
	  }
      System.out.println("End="+stData.tsEnd.toString());

//	if (stData != null)
//	{

String start = new String("2014-08-20 16:00:00.0");
String send  = new String("2014-08-20 16:01:00.0");

      try
      {
        lstart = dfm.parse(start).getTime();
        lend   = dfm.parse(send).getTime();
	  }
      catch (ParseException e)
      {
        e.printStackTrace();
      }

//      diff   = lend-lstart;
//      System.out.println(lstart+" "+lend+" "+diff);

      System.out.println("Start="+stData.tsStart.toString());
      for (i=0; i<=iLogHeight; i+=iIncrementY)
      {
		tsTime = new Timestamp( lstart + (long)i );
		System.out.println(i+" "+tsTime.toString());
	  }
      System.out.println("End="+stData.tsEnd.toString());


//      for (int i=0; i<stData.iRows; i++)
      while (i<stData.iRows)
      {
		if (i==0)
		{
		  str = new String(stData.ts[i].toString());
		  tokens = str.split( sDelimiter );
		  if (tokens.length > 0)
		    sDate = new String(tokens[0].trim());
System.out.println("<"+sDate+">");
//		  date  = dfm.parse(sDate);
          try
          {
            ldate = dfm.parse(sDate).getTime();
          }
          catch (ParseException e)
          {
            e.printStackTrace();
          }
          ts    = new Timestamp(ldate);
System.out.println(ts.toString());
//		  ts    = new Timestamp(date.getTime());
//		  ldate = date.getTime();
		}

//		ldiff = stData.ts[i].getTime() - ldate;
//		tsTime = new Timestamp(ldiff);

//		System.out.println(i+" "+stData.ts[i].toString()+" "+stData.data[i]);
  	    str = new String(stData.ts[i].toString());
		tokens = str.split( sDelimiter );
		if (tokens.length > 0)
		  sTime = new String(tokens[1].trim());

      for (int i=0; i<=iLogHeight; i+=iIncrementY)
      {
//      iDepth = (int) (depthStart +
//                      (double)i * dIncrementDepth / (double) iIncrementY);
		System.out.println(i+" <"+sDate+"><"+sTime+"> "+stData.data[i]);

      }
        i=i+
//		i=i+1000;
	  }

//	}
  }
*/
  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnPC)  { readFile(); }

    if (event.getSource() == btnContinue) { plot(); }   // Plot Data
    if (event.getSource() == btnClear)    { clear(); }  // Clear Data

    setButtons();

    if (event.getSource() == btnExit)     { close(); }    // Exit
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);
  }
}

/** CLASS seisStartFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class seisStartFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 03/06/2014
 *  @author  John Victorine
 */

